#!perl

use strict;
use warnings;

use FindBin;
use lib $FindBin::Bin;

BEGIN { require 'test-helper-common.pl' }

use Shared::Examples::Net::Amazon::S3 (
	qw[ fixture ],
);

use Shared::Examples::Net::Amazon::S3::Request (
	qw[ behaves_like_net_amazon_s3_request ],
);

1;
